mod error;
mod gpodder;
mod r#static;
mod web;

use std::sync::Arc;

use axum::{
    body::Body,
    extract::Request,
    http::StatusCode,
    middleware::Next,
    response::{IntoResponse, Redirect, Response},
    routing::get,
    Router,
};
use http_body_util::BodyExt;
use tower_http::trace::TraceLayer;

#[derive(Clone)]
pub struct Context {
    pub store: ::gpodder::GpodderRepository,
    pub tera: Arc<tera::Tera>,
}

pub fn app(ctx: Context) -> Router {
    Router::new()
        .merge(gpodder::router(ctx.clone()))
        .nest("/static", r#static::router())
        .nest("/_", web::router(ctx.clone()))
        .route("/", get(|| async { Redirect::to("/_") }))
        .layer(axum::middleware::from_fn(header_logger))
        .layer(axum::middleware::from_fn(body_logger))
        .layer(TraceLayer::new_for_http())
        .with_state(ctx)
}

async fn header_logger(request: Request, next: Next) -> Response {
    tracing::debug!("request headers = {:?}", request.headers());

    let res = next.run(request).await;

    tracing::debug!("response headers = {:?}", res.headers());

    res
}

async fn body_logger(request: Request, next: Next) -> Response {
    let (parts, body) = request.into_parts();

    let bytes = match body
        .collect()
        .await
        .map_err(|err| (StatusCode::INTERNAL_SERVER_ERROR, err.to_string()).into_response())
    {
        Ok(res) => res.to_bytes(),
        Err(err) => {
            return err;
        }
    };

    tracing::debug!("request body = {:?}", String::from_utf8(bytes.to_vec()));

    let res = next
        .run(Request::from_parts(parts, Body::from(bytes)))
        .await;

    let (parts, body) = res.into_parts();

    let bytes = match body
        .collect()
        .await
        .map_err(|err| (StatusCode::INTERNAL_SERVER_ERROR, err.to_string()).into_response())
    {
        Ok(res) => res.to_bytes(),
        Err(err) => {
            return err;
        }
    };

    tracing::debug!("response body = {:?}", String::from_utf8(bytes.to_vec()));

    Response::from_parts(parts, Body::from(bytes))
}
