mod page;
mod view;

use std::sync::Arc;

use axum::{
    body::Body,
    http::{HeaderMap, Response, StatusCode},
    response::{Html, IntoResponse},
};

pub use page::Page;
pub use view::View;

const BASE_TEMPLATE: &str = "base.html";

/// Trait defining shared methods for working with typed Tera templates
pub trait Template {
    /// Returns the name or path used to identify the template in the Tera struct
    fn template(&self) -> &'static str;

    /// Render the template using the given Tera instance.
    ///
    /// Templates are expected to manage their own context requirements if needed.
    fn render(&self, tera: &tera::Tera) -> tera::Result<String>;
}

/// Useful additional functions on sized Template implementors
pub trait TemplateExt: Sized + Template {
    fn response(self, tera: &Arc<tera::Tera>) -> TemplateResponse<Self> {
        TemplateResponse::new(tera, self)
    }

    fn page(self, headers: &HeaderMap) -> Page<Self> {
        Page::new(self).headers(headers)
    }
}

impl<T: Sized + Template> TemplateExt for T {}

/// A specific instance of a template. This type can be used as a return type from Axum handlers.
pub struct TemplateResponse<T> {
    tera: Arc<tera::Tera>,
    template: T,
}

impl<T> TemplateResponse<T> {
    pub fn new(tera: &Arc<tera::Tera>, template: T) -> Self {
        Self {
            tera: Arc::clone(&tera),
            template,
        }
    }
}

impl<T: Template> IntoResponse for TemplateResponse<T> {
    fn into_response(self) -> Response<Body> {
        match self.template.render(&self.tera) {
            Ok(s) => Html(s).into_response(),
            Err(err) => {
                tracing::error!("tera template failed: {err}");

                StatusCode::INTERNAL_SERVER_ERROR.into_response()
            }
        }
    }
}

pub fn initialize_tera() -> tera::Result<tera::Tera> {
    let mut tera = tera::Tera::default();

    tera.add_raw_templates([
        (BASE_TEMPLATE, include_str!("templates/base.html")),
        (
            View::Index.template(),
            include_str!("templates/views/index.html"),
        ),
    ])?;

    Ok(tera)
}
