use axum::http::{HeaderMap, HeaderValue};

use super::Template;

const HX_REQUEST_HEADER: &str = "HX-Request";
const HX_HISTORY_RESTORE_HEADER: &str = "HX-History-Restore-Request";

/// Overarching template type that conditionally wraps its inner template with the base template if
/// required, as derived from the request headers
pub struct Page<T> {
    template: T,
    wrap_with_base: bool,
}

impl<T: Template> Template for Page<T> {
    fn template(&self) -> &'static str {
        self.template.template()
    }

    fn render(&self, tera: &tera::Tera) -> tera::Result<String> {
        let inner = self.template.render(tera)?;

        if self.wrap_with_base {
            let mut ctx = tera::Context::new();
            ctx.insert("inner", &inner);

            tera.render(super::BASE_TEMPLATE, &ctx)
        } else {
            Ok(inner)
        }
    }
}

impl<T> Page<T> {
    pub fn new(template: T) -> Self {
        Self {
            template,
            wrap_with_base: false,
        }
    }

    pub fn headers(mut self, headers: &HeaderMap) -> Self {
        let is_htmx_req = headers.get(HX_REQUEST_HEADER).is_some();
        let is_hist_restore_req = headers
            .get(HX_HISTORY_RESTORE_HEADER)
            .map(|val| val == HeaderValue::from_static("true"))
            .unwrap_or(false);

        self.wrap_with_base = !is_htmx_req || is_hist_restore_req;

        self
    }
}
